{*******************************************************}
{                                                       }
{       xTool - Component Collection                    }
{                                                       }
{       Copyright (c) 1995 Stefan Bther                }
{                            stefc@fabula.com           }
{*******************************************************}
{
  Smart object for accessing ISAPI, to write the HTML result you can
  use the standard output device with write & writeln functions.

  Please look also for our xTools-Nails function toolkit or other
  components from us.

  You'll find more information at
     http://www.fabula.com

  Any comments and suggestions are welcome; please send to:
     stefc@fabula.com.

Sample library :

library MyDll;

uses
  Windows,SysUtils,Classes,ISAPI,xIsapi;

function GetExtensionVersion( var Ver: THSE_VERSION_INFO ): Bool; stdcall; export;
begin
  Result:=True;
  try
    Ver.dwExtensionVersion := MAKELONG ( HSE_VERSION_MINOR, HSE_VERSION_MAJOR );
    Ver.lpszExtensionDesc  := 'HEW Telefonbuch ISAPI Extension DLL';
  except
    Result := False ;
  end;
end;

function HttpExtensionProc( var ECB: TEXTENSION_CONTROL_BLOCK ): DWORD; stdcall; export;
begin
  try
    with TIsapi.Create( ECB ) do
      try
        writeln('<html><head><title>Test ISAPI</title></head>');
        writeln('<body background="back.gif">');
        writeln('Hello World');
        writeln('</body></html>');
      finally
        Free;
      end;
    Result := HSE_STATUS_SUCCESS;
  except
    Result := HSE_STATUS_ERROR;
  end;
end;

exports
  GetExtensionVersion,
  HttpExtensionProc;

begin
end.

Sample HTML :

<html><head><title>Test</title></head>
<body background="BACK.gif">
<form action="mydll.dll" method="GET">
<input type=image name="ok" src="search.gif">
</form></body></html>
}

unit xIsapi;

interface

uses
  Windows,SysUtils,Classes,ISAPI;

type
  TIsapi = class
  private
    fHandle : ^TEXTENSION_CONTROL_BLOCK;
    fParams : TStrings;
    fPath   : String;
    function GetVariable(const Index: String): String;
    procedure ParamsScan;
  public
    constructor Create( var aHandle: TEXTENSION_CONTROL_BLOCK );
    destructor Destroy; override;

    procedure Transfer(P: PChar; aLen: Integer);

    property Variable[const Index: String]: String read GetVariable;
    property Params: TStrings read fParams;
    property Path: String read fPath;
  end;

var
  IsapiServer : TIsapi;

procedure AssignHtml(var F: TextFile);

implementation

function strToken(var S: String; Seperator: Char): String;
var
  I               : Word;
begin
  I:=Pos(Seperator,S);
  if I<>0 then
  begin
    Result:=System.Copy(S,1,I-1);
    System.Delete(S,1,I);
  end else
  begin
    Result:=S;
    S:='';
  end;
end;


{ TIsapi }

constructor TIsapi.Create( var aHandle: TEXTENSION_CONTROL_BLOCK );
begin
  inherited Create;
  fHandle := @aHandle;
  fParams := TStringList.Create;

  ParamsScan;

  IsapiServer := Self;

  fPath:=StrPas(fHandle.lpszPathTranslated);

  AssignHtml(Output);
  Rewrite(Output);
end;

destructor TIsapi.Destroy;
begin
  Close(Output);
  IsapiServer := nil;
  fParams.Free;
  inherited Destroy;
end;

procedure TIsapi.Transfer(P: PChar; aLen: Integer);
begin
  fHandle.WriteClient(fHandle.ConnID,P,aLen,0);
end;

function TIsapi.GetVariable(const Index: String): String;
var
  aBuffer: array[0..4096] of char;
  aSize: Integer;
begin
  aSize:=Sizeof(aBuffer);
  if fHandle.GetServerVariable(fHandle.ConnID, PChar(Index),@aBuffer,aSize) then
     Result:=StrPas(aBuffer)
  else
     Result:='';
end;

procedure TIsapi.ParamsScan;
var
  S,aToken: String;

  function ConvertString(const S: String): String;
  var
    i: Integer;
  begin
    i:=1;
    Result:='';
    while (i<=Length(S)) do
    begin
      if S[i]='%' then
      begin
        Result:=Result+Chr(StrToint('$'+Copy(S,i+1,2)));
        Inc(i,3);
      end else begin
        if S[i]='+' then
           Result:=Result+' '
        else
           Result:=Result+S[i];
        Inc(i);
      end;
    end;
  end;

begin
  with fHandle^ do
    if AnsiUppercase(StrPas(lpszMethod))='POST' then
       S:=StrPas(lpbData)
    else
       S:=StrPas(lpszQueryString);

  aToken:=strToken(S,'&');
  while aToken<>'' do
  begin
    fParams.Add(ConvertString(aToken));
    aToken:=strToken(S,'&');
  end;
end;

{ a text device for html output }

function HtmlOutput(var F: TTextRec): Integer;
begin
  with F do
  begin
    // PrnString(PrnRec(UserData), PChar(BufPtr), BufPos);
    if IsapiServer<>nil then IsapiServer.Transfer(PChar(BufPtr), BufPos);
    BufPos := 0;
    Result := 0;
  end;
end;

{ Will ignore certain requests by the I/O system such as flush while doing an
  input. }
function HtmlIgnore(var F: TTextRec): Integer;
begin
  Result := 0;
end;

function HtmlClose(var F: TTextRec): Integer;
begin
  Result:=0;
end;

function HtmlInput(var F: TTextRec): Integer;
begin
  with F do
  begin
    BufPos := 0;
    BufEnd := 0;
  end;
  Result := 0;
end;

function HtmlOpen(var F: TTextRec): Integer;
begin
  with F do begin
    if Mode = fmInput then
    begin
      InOutFunc := @HtmlInput;
      FlushFunc := @HtmlIgnore;
      CloseFunc := @HtmlIgnore;
    end else
    begin
      Mode := fmOutput;
      InOutFunc := @HtmlOutput;
      FlushFunc := @HtmlOutput;
      CloseFunc := @HtmlClose;
    end;
    Result := 0;
  end;
end;

procedure AssignHtml(var F: TextFile);
begin
  with TTextRec(F) do
  begin
    FillChar(F, SizeOf(F), 0);
    Mode := fmClosed;
    BufSize := SizeOf(Buffer);
    BufPtr := @Buffer;
    OpenFunc := @HtmlOpen;
  end;
end;

end.
